"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.get = void 0;
const tslib_1 = require("tslib");
const get = ({ services }) => (router) => {
    const t = router.t;
    const Request = t.Object(t.prop('id', t.Ref('BlockId')).options({
        title: 'Block ID',
        description: 'The ID of the block to retrieve.',
    }));
    const Response = t.Object(t.prop('block', t.Ref('Block').options({})), t.prop('patches', t.Array(t.Ref('BlockPatch'))).options({
        title: 'Patches',
        description: 'The list of all patches.',
    }));
    const Func = t
        .Function(Request, Response)
        .options({
        title: 'Read Block',
        intro: 'Retrieves a block by ID.',
        description: 'Fetches a block by ID.',
    })
        .implement(({ id }) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const { block, patches } = yield services.blocks.get(id);
        return {
            block,
            patches,
        };
    }));
    return router.fn('blocks.get', Func);
};
exports.get = get;
