import { routes } from './routes';
import { system } from './system';
import { TypeRouter } from '../../json-type/system/TypeRouter';
import { TypeRouterCaller } from '../../reactive-rpc/common/rpc/caller/TypeRouterCaller';
import { RpcError } from '../../reactive-rpc/common/rpc/caller';
import { Value } from '../../reactive-rpc/common/messages/Value';
export const createRouter = (services) => {
    const router = new TypeRouter({ system, routes: {} });
    const deps = { services, router };
    return routes(deps)(router);
};
export const createCaller = (services) => {
    const router = createRouter(services);
    const caller = new TypeRouterCaller({
        router,
        wrapInternalError: (error) => {
            if (error instanceof Value)
                return error;
            if (error instanceof RpcError)
                return RpcError.value(error);
            console.error(error);
            return RpcError.valueFrom(error);
        },
    });
    return { router, caller };
};
