"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.publish = void 0;
const tslib_1 = require("tslib");
const publish = ({ services }) => (router) => {
    const t = router.t;
    const req = t.Object(t.prop('channel', t.str).options({
        title: 'Channel name',
        description: 'The name of the channel to publish to.',
    }), t.prop('message', t.any).options({
        title: 'Message',
        description: 'The message to publish to the channel. Can be any JSON/CBOR value.',
    }));
    const res = t.obj.options({
        title: 'Publish response',
        description: 'An empty object.',
    });
    const func = t
        .Function(req, res)
        .options({
        title: 'Publish to channel',
        intro: 'Publish a message to a channel.',
        description: 'This method publishes a message to a global channel with the given `channel` name. ' +
            'All subscribers to the channel will receive the message. The `message` can be any value. ' +
            'The most efficient way to publish a message is to send a primitive or a `Uint8Array` buffer.',
    })
        .implement(({ channel, message }) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        services.pubsub.publish(channel, message);
        return {};
    }));
    return router.fn('pubsub.publish', func);
};
exports.publish = publish;
